"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PdfDataExtractor = void 0;
const pdf_1 = require("pdfjs-dist/es5/build/pdf");
const types_1 = require("./types");
async function getPageNumber(pdf_document, pageRef, cache) {
    const ref = pageRef.gen === 0 ? `${pageRef.num}R` : `${pageRef.num}R${pageRef.gen}`;
    let number = cache[ref];
    if (number == null) {
        number = await pdf_document.getPageIndex(pageRef);
        cache[ref] = number;
    }
    return number;
}
async function parseOutline(pdf_document, outlineData, cache) {
    const outline = [];
    for (const o of outlineData) {
        const dest = o.dest ? o.dest[0] : null;
        if (dest == null) {
            if (o.unsafeUrl != null) {
                outline.push({
                    title: o.title,
                    url: o.unsafeUrl,
                    childs: o.items ? await parseOutline(pdf_document, o.items, cache) : undefined
                });
            }
        }
        else {
            outline.push({
                title: o.title,
                page: await getPageNumber(pdf_document, dest, cache),
                childs: o.items ? await parseOutline(pdf_document, o.items, cache) : undefined
            });
        }
    }
    return outline;
}
/**
 * the extractor for the data of the pdf
 */
class PdfDataExtractor {
    constructor(pdf_document) {
        this.pdf_document = pdf_document;
    }
    /**
     * get the extractor for the data
     *
     * @param {Uint8Array} data - the binary data file
     * @param {PdfDataExtractorOptions} [options={}] - the options on how to open the data in the extractor
     * @returns {Promise<PdfDataExtractor>} a promise that is resolved with a {PdfDataExtractor} object to pull the extracted data from
     */
    static async get(data, options = {}) {
        var _a;
        const pdf_document = await pdf_1.getDocument({
            data: data,
            password: options.password,
            verbosity: (_a = options.verbosity) !== null && _a !== void 0 ? _a : types_1.VerbosityLevel.ERRORS,
        }).promise;
        return new PdfDataExtractor(pdf_document);
    }
    /**
     * get the fingerprint
     *
     * @returns {string} the fingerprint
     */
    get fingerprint() {
        return this.pdf_document.fingerprint;
    }
    /**
     * get the number of pages
     *
     * @returns {string} the number of pages
     */
    get pages() {
        return this.pdf_document.numPages;
    }
    /**
     * get the permission flags
     *
     * @returns {Promise<Permissions | null>} a promise that is resolved with a {Permissions | null} object that contains the permission flags for the PDF
     */
    async getPermissions() {
        const permission_flag_array = await this.pdf_document.getPermissions();
        return permission_flag_array == null ? null : {
            assemble: permission_flag_array.includes(pdf_1.PermissionFlag.ASSEMBLE),
            copy: permission_flag_array.includes(pdf_1.PermissionFlag.COPY),
            copyForAccessibility: permission_flag_array.includes(pdf_1.PermissionFlag.COPY_FOR_ACCESSIBILITY),
            fillInteractiveForms: permission_flag_array.includes(pdf_1.PermissionFlag.FILL_INTERACTIVE_FORMS),
            modifyAnnotations: permission_flag_array.includes(pdf_1.PermissionFlag.MODIFY_ANNOTATIONS),
            print: permission_flag_array.includes(pdf_1.PermissionFlag.PRINT),
            printHQ: permission_flag_array.includes(pdf_1.PermissionFlag.PRINT_HIGH_QUALITY),
            modifyContents: permission_flag_array.includes(pdf_1.PermissionFlag.MODIFY_CONTENTS),
        };
    }
    /**
     * get the text
     *
     * @param {number} [max] - the number of pages to be read
     * @param {boolean} [sort] - sort the text by text coordinates
     * @returns {Promise<string[]>} a promise that is resolved with a {string[]} array with the extracted text per page
     */
    async getText(max, sort) {
        const counter = max && max > 0 ?
            (max > this.pdf_document.numPages ?
                this.pdf_document.numPages
                : max)
            : this.pdf_document.numPages;
        const text_array = [];
        for (let i = 1; i <= counter; i++) {
            const page = await this.pdf_document.getPage(i).catch(() => null);
            const pageText = page == null ? '' : await page.getTextContent().then((textContent) => {
                /*
                    transform is a array with a transform matrix [scale x,shear x,shear y,scale y,offset x, offset y]

                    0,1         1,1
                      -----------
                      |         |
                      |         |
                      |   pdf   |
                      |         |
                      |         |
                      -----------
                    0,0         1,0
                */
                //coordinate based sorting
                if (sort)
                    textContent.items.sort((e1, e2) => {
                        if (e1.transform[5] < e2.transform[5])
                            return 1;
                        else if (e1.transform[5] > e2.transform[5])
                            return -1;
                        else if (e1.transform[4] < e2.transform[4])
                            return -1;
                        else if (e1.transform[4] > e2.transform[4])
                            return 1;
                        else
                            return 0;
                    });
                let lastLineY, text = '';
                for (const item of textContent.items) {
                    if (!lastLineY || lastLineY == item.transform[5]) {
                        text += item.str;
                    }
                    else {
                        text += '\n' + item.str;
                    }
                    lastLineY = item.transform[5];
                }
                return text;
            }, () => '');
            text_array.push(pageText);
        }
        return text_array;
    }
    /**
     * get the outline/bookmarks
     *
     * @returns {Promise<Outline[]>} a promise that is resolved with a {Outline[]} array with information from the tree outline
     */
    async getOutline() {
        return parseOutline(this.pdf_document, await this.pdf_document.getOutline(), {});
    }
    /**
     * get the metadata
     *
     * @returns {Promise<{ info: Info; metadata: Metadata; } | null>} a promise that is resolved with a {{ info: Info; metadata: Metadata; } | null} object with information from the metadata section
     */
    async getMetadata() {
        return await this.pdf_document.getMetadata().catch(() => null);
    }
    /**
     * close the extractor
     *
     * @returns {Promise<void>} a promise that is resolved when destruction is completed
     */
    async close() {
        return this.pdf_document.destroy();
    }
}
exports.PdfDataExtractor = PdfDataExtractor;
//# sourceMappingURL=pdfdataextractor.js.map